@@ -1,7 +1,7 @@
 /*
  *  handle.c
  *
- *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -49,7 +49,6 @@ pmhandle_t *_alpm_handle_new()
 	ALPM_LOG_FUNC;
 
 	CALLOC(handle, 1, sizeof(pmhandle_t), RET_ERR(PM_ERR_MEMORY, NULL));
-	handle->lckfd = -1;
 
 	return(handle);
 }
@@ -232,6 +231,15 @@ int SYMEXPORT alpm_option_get_usedelta()
 	return handle->usedelta;
 }
 
+int SYMEXPORT alpm_option_get_checkspace()
+{
+	if (handle == NULL) {
+		pm_errno = PM_ERR_HANDLE_NULL;
+		return -1;
+	}
+	return handle->checkspace;
+}
+
 pmdb_t SYMEXPORT *alpm_option_get_localdb()
 {
 	if (handle == NULL) {
@@ -550,4 +558,9 @@ void SYMEXPORT alpm_option_set_usedelta(int usedelta)
 	handle->usedelta = usedelta;
 }
 
+void SYMEXPORT alpm_option_set_checkspace(int checkspace)
+{
+	handle->checkspace = checkspace;
+}
+
 /* vim: set ts=2 sw=2 noet: */
