@@ -1,7 +1,7 @@
 /*
  *  conf.h
  *
- *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -111,8 +111,8 @@ enum {
 
 /* clean method */
 enum {
-	PM_CLEAN_KEEPINST = 0, /* default */
-	PM_CLEAN_KEEPCUR
+	PM_CLEAN_KEEPINST = 1,
+	PM_CLEAN_KEEPCUR = (1 << 1)
 };
 
 /* global config variable */
