@@ -637,25 +637,37 @@ static int process_group(alpm_list_t *dbs, char *group)
 		return(1);
 	}
 
-	printf(_(":: There are %d members in group %s:\n"), count,
-			group);
-	select_display(pkgs);
-	char *array = malloc(count);
-	multiselect_question(array, count);
-	int n = 0;
-	for(i = pkgs; i; i = alpm_list_next(i)) {
-		if(array[n++] == 0)
-			continue;
-		pmpkg_t *pkg = alpm_list_getdata(i);
 
-		if(process_pkg(pkg) == 1) {
-			ret = 1;
-			goto cleanup;
+	if(config->print == 0) {
+		printf(_(":: There are %d members in group %s:\n"), count,
+				group);
+		select_display(pkgs);
+		char *array = malloc(count);
+		multiselect_question(array, count);
+		int n = 0;
+		for(i = pkgs; i; i = alpm_list_next(i)) {
+			if(array[n++] == 0)
+				continue;
+			pmpkg_t *pkg = alpm_list_getdata(i);
+
+			if(process_pkg(pkg) == 1) {
+				ret = 1;
+				free(array);
+				goto cleanup;
+			}
+		}
+	} else {
+		for(i = pkgs; i; i = alpm_list_next(i)) {
+			pmpkg_t *pkg = alpm_list_getdata(i);
+
+			if(process_pkg(pkg) == 1) {
+				ret = 1;
+				goto cleanup;
+			}
 		}
 	}
 cleanup:
 	alpm_list_free(pkgs);
-	free(array);
 	return(ret);
 }
 
@@ -663,6 +675,13 @@ static int process_targname(alpm_list_t *dblist, char *targname)
 {
 	pmpkg_t *pkg = alpm_find_dbs_satisfier(dblist, targname);
 
+	/* #FS23342 - skip ignored packages when user says no */
+	if(pm_errno == PM_ERR_PKG_IGNORED) {
+			pm_printf(PM_LOG_WARNING, _("skipping target: %s\n"), targname);
+			pm_errno = 0;
+			return(0);
+	}
+
 	if(pkg) {
 		return(process_pkg(pkg));
 	}
