@@ -16,8 +16,8 @@
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-import sys
 import os
+import re
 import hashlib
 import stat
 
@@ -25,13 +25,10 @@ import stat
 # ALPM
 PM_ROOT     = "/"
 PM_DBPATH   = "var/lib/pacman"
+PM_SYNCDBPATH = "var/lib/pacman/sync"
 PM_LOCK     = "var/lib/pacman/db.lck"
 PM_CACHEDIR = "var/cache/pacman/pkg"
 PM_EXT_PKG  = ".pkg.tar.gz"
-PM_EXT_DB   = ".db.tar.gz"
-PM_PACNEW   = ".pacnew"
-PM_PACORIG  = ".pacorig"
-PM_PACSAVE  = ".pacsave"
 
 # Pacman
 PACCONF     = "etc/pacman.conf"
@@ -89,11 +86,8 @@ def mkfile(name, data = ""):
         path = filename
     else:
         path = os.path.dirname(filename)
-    try:
-        if path and not os.path.isdir(path):
-            os.makedirs(path, 0755)
-    except:
-        error("mkfile: could not create directory hierarchy '%s'" % path)
+    if path and not os.path.isdir(path):
+        os.makedirs(path, 0755)
 
     if isdir:
         return
@@ -113,52 +107,6 @@ def mkfile(name, data = ""):
         if setperms:
             os.chmod(filename, int(perms, 8))
 
-def mkdescfile(filename, pkg):
-    """
-    """
-
-    data = []
-
-    # desc
-    #data.append("pkgname = %s" % pkg.name)
-    #data.append("pkgver = %s" % pkg.version)
-    if pkg.desc:
-        data.append("pkgdesc = %s" % pkg.desc)
-    if pkg.url:
-        data.append("url = %s" % pkg.url)
-    if pkg.builddate:
-        data.append("builddate = %s" % pkg.builddate)
-    if pkg.packager:
-        data.append("packager = %s" % pkg.packager)
-    if pkg.size:
-        data.append("size = %s" % pkg.size)
-    if pkg.arch:
-        data.append("arch = %s" % pkg.arch)
-    for i in pkg.groups:
-        data.append("group = %s" % i)
-    for i in pkg.license:
-        data.append("license = %s" % i)
-    if pkg.md5sum:
-        data.append("md5sum = %s" % pkg.md5sum)
-
-    # depends
-    for i in pkg.replaces:
-        data.append("replaces = %s" % i)
-    for i in pkg.depends:
-        data.append("depend = %s" % i)
-    for i in pkg.optdepends:
-        data.append("optdepend = %s" % i)
-    for i in pkg.conflicts:
-        data.append("conflict = %s" % i)
-    for i in pkg.provides:
-        data.append("provides = %s" % i)
-    for i in pkg.backup:
-        data.append("backup = %s" % i)
-    if pkg.force:
-        data.append("force = 1")
-
-    mkfile(filename, "\n".join(data))
-
 def mkinstallfile(filename, install):
     """
     """
@@ -178,8 +126,11 @@ def mkcfgfile(filename, root, option, db):
         data.extend(["%s = %s" % (key, j) for j in value])
 
     # Repositories
-    for key, value in db.iteritems():
+    # sort by repo name so tests can predict repo order, rather than be
+    # subjects to the whims of python dict() ordering
+    for key in sorted(db.iterkeys()):
         if key != "local":
+            value = db[key]
             data.append("[%s]\n" \
                     "Server = file://%s" \
                      % (value.treename,
@@ -203,21 +154,19 @@ def getmd5sum(filename):
     fd = open(filename, "rb")
     checksum = hashlib.md5()
     while 1:
-        block = fd.read(1048576)
+        block = fd.read(32 * 1024)
         if not block:
             break
         checksum.update(block)
     fd.close()
-    digest = checksum.digest()
-    return "%02x"*len(digest) % tuple(map(ord, digest))
+    return checksum.hexdigest()
 
 def mkmd5sum(data):
     """
     """
     checksum = hashlib.md5()
     checksum.update("%s\n" % data)
-    digest = checksum.digest()
-    return "%02x"*len(digest) % tuple(map(ord, digest))
+    return checksum.hexdigest()
 
 
 #
@@ -233,12 +182,6 @@ def getmtime(filename):
     st = os.stat(filename)
     return st[stat.ST_ATIME], st[stat.ST_MTIME], st[stat.ST_CTIME]
 
-def diffmtime(mt1, mt2):
-    """ORE: TBD
-    """
-    return not mt1 == mt2
-
-
 #
 # Miscellaneous
 #
@@ -252,25 +195,20 @@ def which(filename):
     return None
 
 def grep(filename, pattern):
-    lines = file(filename, 'r').readlines()
-    for line in lines:
-        if not line: break
-        if line.find(pattern) != -1:
+    pat = re.compile(pattern)
+    myfile = open(filename, 'r')
+    for line in myfile:
+        if pat.search(line):
+            myfile.close()
             return True
+    myfile.close()
     return False
 
-def mkdir(dir):
-    if os.path.isdir(dir):
+def mkdir(path):
+    if os.path.isdir(path):
         return
-    elif os.path.isfile(dir):
-        raise OSError("'%s' already exists and is not a directory" % dir)
-    else:
-        parent, thisdir = os.path.split(dir)
-        if parent: mkdir(parent) #recurse to make all parents
-        vprint("making dir %s" % thisdir)
-        if thisdir: os.mkdir(dir)
-
-if __name__ == "__main__":
-    pass
+    elif os.path.isfile(path):
+        raise OSError("'%s' already exists and is not a directory" % path)
+    os.makedirs(path, 0755)
 
 # vim: set ts=4 sw=4 et:
